@@ -83,6 +83,7 @@ data PortID =
 #if !defined(mingw32_HOST_OS) && !defined(cygwin32_HOST_OS) && !defined(_WIN32)
         | UnixSocket String             -- Unix family socket in file system
 #endif
+        deriving (Show, Eq)
 
 -- | Calling 'connectTo' creates a client side socket which is
 -- connected to the given host and port.  The Protocol and socket type is
@@ -161,17 +162,21 @@ connect' host serv = do
 -- | Creates the server side socket which has been bound to the
 -- specified port.
 --
--- NOTE: To avoid the \"Address already in use\"
--- problems popped up several times on the GHC-Users mailing list we
--- set the 'ReuseAddr' socket option on the listening socket.  If you
--- don't want this behaviour, please use the lower level
--- 'Network.Socket.listen' instead.
+-- 'maxListenQueue' (typically 128) is specified to the listen queue.
+-- This is good enough for normal network servers but is too small
+-- for high performance servers.
+--
+-- To avoid the \"Address already in use\" problems,
+-- the 'ReuseAddr' socket option is set on the listening socket.
 --
 -- If available, the 'IPv6Only' socket option is set to 0
 -- so that both IPv4 and IPv6 can be accepted with this socket.
+--
+-- If you don't like the behavior above, please use the lower level
+-- 'Network.Socket.listen' instead.
 
 listenOn :: PortID      -- ^ Port Identifier
-         -> IO Socket   -- ^ Connected Socket
+         -> IO Socket   -- ^ Listening Socket
 
 #if defined(IPV6_SOCKET_SUPPORT)
 -- IPv6 and IPv4.
@@ -464,5 +469,5 @@ firstSuccessful :: [IO a] -> IO a
 firstSuccessful [] = error "firstSuccessful: empty list"
 firstSuccessful (p:ps) = catchIO p $ \e ->
     case ps of
-        [] -> Exception.throw e
+        [] -> Exception.throwIO e
         _  -> firstSuccessful ps
