@@ -130,15 +130,11 @@ import Data.Bits ((.|.),(.&.),shiftL,shiftR)
 import Debug.Trace (trace)
 import Numeric (showIntAtBase)
 
-#ifdef __GLASGOW_HASKELL__
 import Data.Typeable (Typeable)
-# if MIN_VERSION_base(4,0,0)
+#if MIN_VERSION_base(4,0,0)
 import Data.Data (Data)
-# else
-import Data.Generics (Data)
-# endif
 #else
-import Data.Typeable (Typeable(..), TyCon, mkTyCon, mkTyConApp)
+import Data.Generics (Data)
 #endif
 
 ------------------------------------------------------------
@@ -160,38 +156,14 @@ data URI = URI
     , uriPath       :: String           -- ^ @\/ghc@
     , uriQuery      :: String           -- ^ @?query@
     , uriFragment   :: String           -- ^ @#frag@
-    } deriving (Eq, Ord
-#ifdef __GLASGOW_HASKELL__
-    , Typeable, Data
-#endif
-    )
-
-#ifndef __GLASGOW_HASKELL__
-uriTc :: TyCon
-uriTc = mkTyCon "URI"
-
-instance Typeable URI where
-  typeOf _ = mkTyConApp uriTc []
-#endif
+    } deriving (Eq, Ord, Typeable, Data)
 
 -- |Type for authority value within a URI
 data URIAuth = URIAuth
     { uriUserInfo   :: String           -- ^ @anonymous\@@
     , uriRegName    :: String           -- ^ @www.haskell.org@
     , uriPort       :: String           -- ^ @:42@
-    } deriving (Eq, Ord, Show
-#ifdef __GLASGOW_HASKELL__
-    , Typeable, Data
-#endif
-    )
-
-#ifndef __GLASGOW_HASKELL__
-uriAuthTc :: TyCon
-uriAuthTc = mkTyCon "URIAuth"
-
-instance Typeable URIAuth where
-  typeOf _ = mkTyConApp uriAuthTc []
-#endif
+    } deriving (Eq, Ord, Show, Typeable, Data)
 
 -- |Blank URI
 nullURI :: URI
@@ -494,7 +466,7 @@ ipvFuture =
         ; h <- hexDigitChar
         ; char '.'
         ; a <- many1 (satisfy isIpvFutureChar)
-        ; return $ 'c':h:'.':a
+        ; return $ 'v':h:'.':a
         }
 
 isIpvFutureChar :: Char -> Bool
@@ -970,9 +942,50 @@ escapeURIString p s = concatMap (escapeURIChar p) s
 --
 unEscapeString :: String -> String
 unEscapeString [] = ""
-unEscapeString ('%':x1:x2:s) | isHexDigit x1 && isHexDigit x2 =
-    chr (digitToInt x1 * 16 + digitToInt x2) : unEscapeString s
-unEscapeString (c:s) = c : unEscapeString s
+unEscapeString s@(c:cs) = case unEscapeByte s of
+    Just (byte, rest) -> unEscapeUtf8 byte rest
+    Nothing -> c : unEscapeString cs
+
+unEscapeByte :: String -> Maybe (Int, String)
+unEscapeByte ('%':x1:x2:s) | isHexDigit x1 && isHexDigit x2 =
+    Just (digitToInt x1 * 16 + digitToInt x2, s)
+unEscapeByte _ = Nothing
+
+-- Adapted from http://hackage.haskell.org/package/utf8-string
+-- by Eric Mertens, BSD3
+unEscapeUtf8 :: Int -> String -> String
+unEscapeUtf8 c rest
+    | c < 0x80 = chr c : unEscapeString rest
+    | c < 0xc0 = replacement_character : unEscapeString rest
+    | c < 0xe0 = multi1
+    | c < 0xf0 = multi_byte 2 0xf 0x800
+    | c < 0xf8 = multi_byte 3 0x7 0x10000
+    | c < 0xfc = multi_byte 4 0x3 0x200000
+    | c < 0xfe = multi_byte 5 0x1 0x4000000
+    | otherwise    = replacement_character : unEscapeString rest
+    where
+    replacement_character = '\xfffd'
+    multi1 = case unEscapeByte rest of
+      Just (c1, ds) | c1 .&. 0xc0 == 0x80 ->
+        let d = ((fromEnum c .&. 0x1f) `shiftL` 6) .|.  fromEnum (c1 .&. 0x3f)
+        in if d >= 0x000080 then toEnum d : unEscapeString ds
+                            else replacement_character : unEscapeString ds
+      _ -> replacement_character : unEscapeString rest
+
+    multi_byte i mask overlong =
+      aux i rest (unEscapeByte rest) (c .&. mask)
+      where
+        aux 0 rs _ acc
+          | overlong <= acc && acc <= 0x10ffff &&
+            (acc < 0xd800 || 0xdfff < acc)     &&
+            (acc < 0xfffe || 0xffff < acc)      = chr acc : unEscapeString rs
+          | otherwise = replacement_character : unEscapeString rs
+
+        aux n _ (Just (r, rs)) acc
+          | r .&. 0xc0 == 0x80 = aux (n-1) rs (unEscapeByte rs)
+                               $! shiftL acc 6 .|. (r .&. 0x3f)
+
+        aux _ rs _ _ = replacement_character : unEscapeString rs
 
 ------------------------------------------------------------
 -- Resolving a relative URI relative to a base URI
