@@ -13,12 +13,9 @@ initWinSock ()
   WORD wVersionRequested;
   WSADATA wsaData;  
   int err;
-#ifdef __HUGS__
-  int optval = SO_SYNCHRONOUS_NONALERT;
-#endif
 
   if (!winsock_inited) {
-    wVersionRequested = MAKEWORD( 1, 1 );
+    wVersionRequested = MAKEWORD( 2, 2 );
 
     err = WSAStartup ( wVersionRequested, &wsaData );
     
@@ -26,19 +23,11 @@ initWinSock ()
        return err;
     }
 
-    if ( LOBYTE( wsaData.wVersion ) != 1 ||
-       HIBYTE( wsaData.wVersion ) != 1 ) {
+    if ( LOBYTE( wsaData.wVersion ) != 2 ||
+       HIBYTE( wsaData.wVersion ) != 2 ) {
       WSACleanup();
       return (-1);
     }
-#ifdef __HUGS__
-    /* By default, socket() creates sockets in overlapped mode
-     * (so that async I/O is possible). The CRT can only handle
-     * non-overlapped sockets, so turn off overlap mode here.
-     */
-    setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
-	       &optval, sizeof(optval));
-#endif
 
     winsock_inited = 1;
   }
