@@ -50,14 +50,11 @@
 extern void  shutdownWinSock();
 extern int   initWinSock ();
 extern const char* getWSErrorDescr(int err);
-
-# if !defined(__HUGS__)
 extern void* newAcceptParams(int sock,
 			     int sz,
 			     void* sockaddr);
 extern int   acceptNewSock(void* d);
 extern int   acceptDoProc(void* param);
-# endif
 
 #else
 
@@ -175,4 +172,8 @@ hsnet_freeaddrinfo(struct addrinfo *ai)
 # endif
 #endif
 
+#if !defined(IOV_MAX)
+# define IOV_MAX 1024
+#endif
+
 #endif /* HSNET_H */
