@@ -16,6 +16,15 @@
 --
 --  This Module contains test cases for module URI.
 --
+--  To run this test without using Cabal to build the package
+--  (2013-01-05, instructions tested on MacOS):
+--  1. Install Haskell platform
+--  2. cabal install test-framework
+--  3. cabal install test-framework-hunit
+--  4. ghc -XDeriveDataTypeable -D"MIN_VERSION_base(x,y,z)=1" ../Network/URI.hs uri001.hs
+--  5. ./uri001
+--
+--  Previous build instructions:
 --  Using GHC, I compile with this command line:
 --  ghc --make -fglasgow-exts
 --      -i..\;C:\Dev\Haskell\Lib\HUnit;C:\Dev\Haskell\Lib\Parsec
@@ -47,6 +56,7 @@ import Data.Maybe (fromJust)
 import System.IO (openFile, IOMode(WriteMode), hClose)
 import qualified Test.Framework as TF
 import qualified Test.Framework.Providers.HUnit as TF
+import qualified Test.Framework.Providers.QuickCheck2 as TF
 
 -- Test supplied string for valid URI reference syntax
 --   isValidURIRef :: String -> Bool
@@ -220,6 +230,22 @@ testURIRef118 = testURIRef AbsId "http://192.168.0.1.example.com/"
 testURIRef119 = testURIRef AbsId "http://192.168.0.1.example.com./"
 -- URI prefixed with 3 octets of an IPv4 address and a subdomain part with a leading digit.
 testURIRef120 = testURIRef AbsId "http://192.168.0.1test.example.com/"
+-- URI with IPv(future) address
+testURIRef121 = testURIRef AbsId "http://[v9.123.abc;456.def]/"
+testURIRef122 = testEq "v.future authority" 
+                       (Just (URIAuth "" "[v9.123.abc;456.def]" ":42"))
+                       ((maybe Nothing uriAuthority) . parseURI $ "http://[v9.123.abc;456.def]:42/") 
+-- URI with non-ASCII characters, fail with Network.HTTP escaping code (see below)
+-- Currently not supported by Network.URI, but captured here for possible future reference
+-- when IRI support may be added.
+testURIRef123 = testURIRef AbsId "http://example.com/test123/䡥汬漬⁗潲汤/index.html"
+testURIRef124 = testURIRef AbsId "http://example.com/test124/Москва/index.html"
+
+-- From report by Alexander Ivanov:
+-- should return " 䡥汬漬⁗潲汤", but returns "Hello, World" instead
+-- print $ urlDecode $ urlEncode " 䡥汬漬⁗潲汤"
+-- should return "Москва"
+-- print $ urlDecode $ urlEncode "Москва"
 
 testURIRefSuite = TF.testGroup "Test URIrefs" testURIRefList
 testURIRefList =
@@ -344,6 +370,12 @@ testURIRefList =
   , TF.testCase "testURIRef118" testURIRef118
   , TF.testCase "testURIRef119" testURIRef119
   , TF.testCase "testURIRef120" testURIRef120
+    --
+  , TF.testCase "testURIRef121" testURIRef121
+  , TF.testCase "testURIRef122" testURIRef122
+    -- IRI test cases not currently supported
+  -- , TF.testCase "testURIRef123" testURIRef123
+  -- , TF.testCase "testURIRef124" testURIRef124
   ]
 
 -- test decomposition of URI into components
@@ -1072,6 +1104,12 @@ testEscapeURIString06 = testEq "testEscapeURIString06"
     "hello%C3%B8%C2%A9%E6%97%A5%E6%9C%AC"
     (escapeURIString isUnescapedInURIComponent "helloø©日本")
 
+propEscapeUnEscapeLoop :: String -> Bool
+propEscapeUnEscapeLoop s = s == (unEscapeString $! escaped)
+	where
+	escaped = escapeURIString (const False) s
+	{-# NOINLINE escaped #-}
+
 testEscapeURIString = TF.testGroup "testEscapeURIString"
   [ TF.testCase "testEscapeURIString01" testEscapeURIString01
   , TF.testCase "testEscapeURIString02" testEscapeURIString02
@@ -1079,6 +1117,7 @@ testEscapeURIString = TF.testGroup "testEscapeURIString"
   , TF.testCase "testEscapeURIString04" testEscapeURIString04
   , TF.testCase "testEscapeURIString05" testEscapeURIString05
   , TF.testCase "testEscapeURIString06" testEscapeURIString06
+  , TF.testProperty "propEscapeUnEscapeLoop" propEscapeUnEscapeLoop
   ]
 
 -- URI string normalization tests
